#include <msp/crypto/md5.h>
#include <msp/test/test.h>

using namespace std;
using namespace Msp;

class MD5Tests: public Test::RegisteredTest<MD5Tests>
{
public:
	MD5Tests();

	static const char *get_name() { return "MD5"; }

private:
	void empty_input();
	void known_strings();
	void size_corner_cases();
	void large_input();
};

MD5Tests::MD5Tests()
{
	add(&MD5Tests::empty_input,       "empty input");
	add(&MD5Tests::known_strings,     "known strings");
	add(&MD5Tests::size_corner_cases, "input size corner cases");
	add(&MD5Tests::large_input,       "large input");
}

void MD5Tests::empty_input()
{
	string digest = Crypto::MD5().get_hexdigest();
	EXPECT_EQUAL(digest, "d41d8cd98f00b204e9800998ecf8427e");
}

void MD5Tests::known_strings()
{
	// Test suite from RFC 1321
	const char *test_suite[] =
	{
		"a", "0cc175b9c0f1b6a831c399e269772661",
		"abc", "900150983cd24fb0d6963f7d28e17f72",
		"message digest", "f96b697d7cb7938d525a2f31aaf161d0",
		"abcdefghijklmnopqrstuvwxyz", "c3fcd3d76192e4007dfb496cca67e13b",
		"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789", "d174ab98d277d9f5a5611c2c9f419d9f",
		"12345678901234567890123456789012345678901234567890123456789012345678901234567890", "57edf4a22be3c955ac49da2e2107b67a",
		0
	};

	for(unsigned i=0; test_suite[i]; i+=2)
	{
		string digest = Crypto::MD5(test_suite[i]).get_hexdigest();
		expect_equal(digest, digest==test_suite[i+1], string("digest == \"")+test_suite[i+1]+"\"");
	}

	// This appears in Wikipedia
	string digest = Crypto::MD5("The quick brown fox jumps over the lazy dog").get_hexdigest();
	EXPECT_EQUAL(digest, "9e107d9d372bb6826bd81d3542a419d6");
}

void MD5Tests::size_corner_cases()
{
	// 55 bytes; shortest possible padding
	string digest = Crypto::MD5("0123456789abcdef0123456789abcdef0123456789abcdef0123456").get_hexdigest();
	EXPECT_EQUAL(digest, "d8ea71eb4d2af27f59a5316c971065e6");

	// 56 bytes; longest possible padding
	digest = Crypto::MD5("0123456789abcdef0123456789abcdef0123456789abcdef01234567").get_hexdigest();
	EXPECT_EQUAL(digest, "a68f061e81239660f6305195739ba7f0");

	// 64 bytes; exactly one block
	digest = Crypto::MD5("0123456789abcdef0123456789abcdef0123456789abcdef0123456789abcdef").get_hexdigest();
	EXPECT_EQUAL(digest, "fe3a1ff59f3b89b2ad3d33f08984874b");
}

void MD5Tests::large_input()
{
	static const char lorem_ipsum[] = "Lorem ipsum dolor sit amet, consectetur "
		"adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore "
		"magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation "
		"ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute "
		"irure dolor in reprehenderit in voluptate velit esse cillum dolore eu "
		"fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, "
		"sunt in culpa qui officia deserunt mollit anim id est laborum.";

	string digest = Crypto::MD5(lorem_ipsum).get_hexdigest();
	EXPECT_EQUAL(digest, "db89bb5ceab87f9c0fcc2ab36c189c2c");
}
